﻿using System;
using System.Windows;
using System.Windows.Controls;
using Ozeki.MediaGateway;

namespace _03_Webphone_Silverlight_Web2SIP_calls
{
    public partial class MainPage : UserControl
    {
        private MediaConnection connection;
        private MediaStreamSender streamSender;
        private MediaStreamReceiver streamReceiver;
        private AudioPlayer audioPlayer;
        private Microphone microphone;


        private PhoneState phoneState;

        public void setPhoneState(PhoneState newState)
        {
            switch (newState)
            {
                case PhoneState.Ready:
                    btnCall.IsEnabled = true;
                    btnStopCall.IsEnabled = true;
                    break;
                case PhoneState.InRinging:
                    btnCall.IsEnabled = true;
                    break;
                case PhoneState.OutRinging:
                    btnCall.IsEnabled = false;
                    break;
                case PhoneState.InCall:
                    btnCall.IsEnabled = false;
                    break;
            }

            phoneState = newState;
        }

        public MainPage()
        {
            InitializeComponent();
            connection = new MediaConnection("localhost:4502/SIPGateway");
            connection.ConnectionStateChanged += connection_ConnectionStateChanged;
            connection.Connect();
        }
        /// <summary>
        /// Indicates the connectionstate has been changed with server.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void connection_ConnectionStateChanged(object sender, GenericEventArgs<ConnectionState> e)
        {
            switch (e.Item)
            {
                case ConnectionState.Success:
                    txtboxInfo.Text = "Online";
                    phoneState = PhoneState.Ready;
                    audioPlayer = new AudioPlayer();
                    connection.Client = this;
                    break;
                default:
                    InvokeGUIThread(() =>
                    {
                        txtboxInfo.Text = "Connection failed.";
                        btnStopCall.IsEnabled = btnCall.IsEnabled = false;
                    });
                    break;
            }
        }

        /// <summary>
        /// Start a call or accept a call request.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnCall_Click(object sender, RoutedEventArgs e)
        {
            if (Microphone.GetPermissionToMicrophone())
            {
                ReleaseStreams();

                switch (phoneState)
                {
                    case PhoneState.InRinging:
                        connection.InvokeOnConnection("Accept");
                        setPhoneState(PhoneState.InCall);
                        break;
                    case PhoneState.Ready:
                        connection.InvokeOnConnection("Call", txtDial.Text);
                        txtboxInfo.Text = "Outgoing call progress.";
                        setPhoneState(PhoneState.OutRinging);
                        break;
                }
            }
            else
            {
                txtboxInfo.Text = "Please, add permission to access microphone.";
            }
        }

        /// <summary>
        /// befejezi a folyamatban lévő hívást, vagy a bejövő híjvást elutasítja.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnStopCall_Click(object sender, RoutedEventArgs e)
        {
            switch (phoneState)
            {
                case PhoneState.InRinging:
                    connection.InvokeOnConnection("Reject");
                    setPhoneState(PhoneState.Ready);
                    break;
                case PhoneState.OutRinging:
                case PhoneState.InCall:
                    txtboxInfo.Text = "Call stop, ready to call.";
                    connection.InvokeOnConnection("HangUp");
                    setPhoneState(PhoneState.Ready);
                    break;
            }
            txtDial.Text = "";
        }

        private void btnNumpads_Click(object sender, RoutedEventArgs e)
        {
            Button btn = sender as Button;
            if (btn != null)
            {
                txtDial.Text += btn.Content;
            }
        }

        /// <summary>
        /// Sets the client status. 
        /// </summary>
        /// <param name="isReady"></param>
        /// <param name="name"></param>
        public void OnSetReadyStatus(bool isReady, string name)
        {

            btnStopCall.IsEnabled = isReady;
            btnCall.IsEnabled = isReady;
            txtBClientID.Text = name;
            txtboxInfo.Text = isReady ? "Ready to call." : "Offline";
        }


        /// <summary>
        /// Indicates the incoming call request from the given other client.
        /// </summary>
        /// <param name="remotpartyId"> The request owner's client id.</param>
        public void OnCallRequest(string remotpartyId)
        {
            setPhoneState(PhoneState.InRinging);
            InvokeGUIThread(() => { txtboxInfo.Text = "Call received from " + remotpartyId; });
        }


        /// <summary>
        /// Starts to send the audio data to the server and starts to play other party's media data.
        /// </summary>
        /// <param name="publisStreamName"></param>
        /// <param name="playStreamName"></param>
        public void OnInCall(string publisStreamName, string playStreamName)
        {
            streamSender = new MediaStreamSender(connection);
            microphone = Microphone.GetMicrophone();
            streamSender.AttachMicrophone(microphone);
            streamSender.Publish(publisStreamName);

            streamReceiver = new MediaStreamReceiver(connection);
            streamReceiver.AttachAudioPlayer(audioPlayer);
            streamReceiver.Play(playStreamName);
        }

        /// <summary>
        ///  Indicates an error.
        /// </summary>
        /// <param name="callError"></param>
        public void OnCallErrorOccurred(CallError callError)
        {
            InvokeGUIThread(() =>
            {
                txtboxInfo.Text = callError.ToString();
                setPhoneState(PhoneState.Ready);
            });
        }

        /// <summary>
        /// Indicates the call state has changed.
        /// </summary>
        /// <param name="callState">The current state of the call.</param>
        public void OnCallStateChanged(CallState callState)
        {
            InvokeGUIThread(() =>
            {
                txtboxInfo.Text = callState.ToString();
                switch (callState)
                {
                    case CallState.InCall:
                        setPhoneState(PhoneState.InCall);
                        break;
                    case CallState.Cancelled:
                    case CallState.Rejected:
                    case CallState.Busy:
                        setPhoneState(PhoneState.Ready);
                        txtDial.Text = "";
                        break;
                    case CallState.Completed:
                        setPhoneState(PhoneState.Ready);
                        txtDial.Text = "";
                        ReleaseStreams();
                        break;
                }
            });
        }

        /// <summary>
        /// Release MediaStreamSender or MediaStreamReceiver object if it necessary.(All started media call action required new MediaStreamSender and MediaStreamReciver object.)
        /// </summary>
        private void ReleaseStreams()
        {
            if (streamSender != null)
            {
                streamSender.Close();
            }
            if (streamReceiver != null)
            {
                streamReceiver.Close();
            }
        }


        /// <summary>
        /// The controls of the Windows Form applications can only be modified on the GUI thread. This method grants access to the GUI thread.
        /// </summary>
        /// <param name="action"></param>
        private void InvokeGUIThread(Action action)
        {
            Dispatcher.BeginInvoke(action);
        }

    }
}
